@@ -17,7 +17,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 
-"""Connect engine API"""
+"""Connect to Engine API"""
 
 
 import gettext
@@ -50,7 +50,7 @@ def get_engine_api(
     while not valid and attempts > 0:
         attempts = attempts - 1
         try:
-            base.logger.info(_('Connecting to the Engine'))
+            base.logger.info(_('Connecting to Engine'))
             insecure = False
             if base.environment[
                 ohostedcons.EngineEnv.INSECURE_SSL
@@ -89,13 +89,16 @@ def get_engine_api(
                 if base.environment[
                     ohostedcons.EngineEnv.INTERACTIVE_ADMIN_PASSWORD
                 ]:
-                    base.logger.error(
-                        _(
-                            'The engine API didn''t accepted '
-                            'the administrator password you provided\n'
-                            'Please enter it again to retry.'
+                    if base.environment[
+                        ohostedcons.EngineEnv.ADMIN_PASSWORD
+                    ] is not None:
+                        base.logger.error(
+                            _(
+                                'The Engine API didn''t accept '
+                                'the administrator password you provided.\n'
+                                'Please enter it again to retry.'
+                            )
                         )
-                    )
                     base.environment[
                         ohostedcons.EngineEnv.ADMIN_USERNAME
                     ] = base.dialog.queryString(
@@ -119,14 +122,14 @@ def get_engine_api(
                 else:
                     raise RuntimeError(
                         _(
-                            'The engine API didn''t accepted '
+                            'The Engine API didn''t accept '
                             'the administrator password you provided\n'
                         )
                     )
             else:
                 base.logger.debug(
                     _(
-                        'Cannot connect to engine APIs on {fqdn}:\n'
+                        'Cannot connect to Engine API on {fqdn}:\n'
                         '{details}\n'
                     ).format(
                         fqdn=fqdn,
