@@ -31,6 +31,8 @@ from otopi import constants as otopicons
 from otopi import plugin
 from otopi import util
 
+from vdsm.client import ServerError
+
 from ovirt_setup_lib import dialog
 
 from ovirt_hosted_engine_setup import constants as ohostedcons
@@ -74,13 +76,18 @@ class Plugin(plugin.PluginBase):
                 )
             try:
                 if address:
-                    match = self._IPADDR_RE.match(address)
-                    if match:
-                        # TODO: figure out a better regexp avoiding this check
-                        valid = True
-                        for i in match.groups():
-                            valid &= int(i) >= 0
-                            valid &= int(i) < 255
+                    valid = True
+                    for a in address.split(','):
+                        match = self._IPADDR_RE.match(a)
+                        if match:
+                            # TODO: figure out a better regexp
+                            # avoiding this check
+                            valid &= True
+                            for i in match.groups():
+                                valid &= int(i) >= 0
+                                valid &= int(i) < 255
+                        else:
+                            valid = False
                 if not valid:
                     raise ValueError(_('Address must be a valid IP address'))
             except ValueError as e:
@@ -110,11 +117,12 @@ class Plugin(plugin.PluginBase):
                     default=ohostedcons.Defaults.DEFAULT_ISCSI_PORT,
                 )
             try:
-                int_port = int(port)
-                if int_port > 0 and int_port < 65536:
-                    valid = True
-                else:
-                    raise ValueError(_('Port must be a valid port number'))
+                for p in port.split(','):
+                    int_port = int(p)
+                    if int_port > 0 and int_port < 65536:
+                        valid = True
+                    else:
+                        raise ValueError(_('Port must be a valid port number'))
             except ValueError as e:
                 if self.environment[ohostedcons.StorageEnv.ISCSI_PORT] is None:
                     self.logger.debug('exception', exc_info=True)
@@ -179,28 +187,72 @@ class Plugin(plugin.PluginBase):
             store=False,
         )
 
-    def _customize_target(self, values, default):
+    def _customize_target(self, values):
         target = self.environment[ohostedcons.StorageEnv.ISCSI_TARGET]
         if target is None:
             self._interactive = True
-            target = self.dialog.queryString(
+            f_targets = []
+            for target in values:
+                for tpgt in values[target]:
+                    f_targets.append(
+                        {
+                            'index': str(len(f_targets)+1),
+                            'target': target,
+                            'tpgt': tpgt,
+                        }
+                    )
+            target_list = ''
+            for entry in f_targets:
+                target_list += _(
+                    '\t[{index}]\t{target}\n\t\tTPGT: {tpgt}, portals:\n'
+                ).format(
+                    index=entry['index'],
+                    target=entry['target'],
+                    tpgt=entry['tpgt'],
+                )
+                for portal in values[entry['target']][entry['tpgt']]:
+                    target_list += _(
+                        '\t\t\t{portal}:{port}\n'
+                    ).format(
+                        portal=portal['ip'],
+                        port=portal['port'],
+                    )
+                target_list += '\n'
+
+            self.dialog.note(
+                _(
+                    'The following targets have been found:\n'
+                    '{target_list}'
+                ).format(
+                    target_list=target_list,
+                )
+            )
+
+            s_target = self.dialog.queryString(
                 name='OVEHOSTED_STORAGE_ISCSI_TARGET',
                 note=_(
-                    'Please specify the target name '
+                    'Please select a target '
                     '(@VALUES@) [@DEFAULT@]: '
                 ),
                 prompt=True,
                 caseSensitive=True,
-                default=default,
-                validValues=values,
+                default='1',
+                validValues=[i['index'] for i in f_targets],
             )
-        return target
+        else:
+            tpgt = self.environment[ohostedcons.StorageEnv.ISCSI_PORTAL] or '1'
+            f_targets = [{'target': target, 'tpgt': tpgt}]
+            s_target = 1
+        return (
+            f_targets[int(s_target)-1]['target'],
+            f_targets[int(s_target)-1]['tpgt']
+        )
 
-    def _customize_lun(self, domainType, target):
+    def _customize_lun(self, domainType, target, ip_port_list=None, tpgt=None):
         if domainType == ohostedcons.DomainTypes.ISCSI:
             available_luns = self._iscsi_get_lun_list(
-                ip=self.environment[ohostedcons.StorageEnv.ISCSI_IP_ADDR],
-                port=self.environment[ohostedcons.StorageEnv.ISCSI_PORT],
+                ip_port_list=ip_port_list,
+                tpgt=tpgt,
                 user=self.environment[ohostedcons.StorageEnv.ISCSI_USER],
                 password=self.environment[
                     ohostedcons.StorageEnv.ISCSI_PASSWORD
@@ -215,6 +267,7 @@ class Plugin(plugin.PluginBase):
 
         f_luns = []
         lun_list = ''
+        available_luns = sorted(available_luns, key=lambda lun: lun['GUID'])
         for entry in available_luns:
             activep = 0
             failedp = 0
@@ -299,90 +352,93 @@ class Plugin(plugin.PluginBase):
                 default=_('Abort'),
             ).lower() == _('Force').lower()
 
-    def _create_vg(self, forceVG=False):
-        return self.cli.createVG(
-            name=self.environment[ohostedcons.StorageEnv.SD_UUID],
-            devlist=[
-                self.environment[
-                    ohostedcons.StorageEnv.LUN_ID
-                ],
-            ],
-            force=forceVG,
-        )
-
     def _iscsi_discovery(self, address, port, user, password):
-        targets = self.cli.discoverSendTargets(
-            host=address,
-            port=port,
-            user=user,
-            password=password,
-        )
-        self.logger.debug(targets)
-        if targets['status']['code'] != 0:
-            raise RuntimeError(targets['status']['message'])
+        try:
+            targets = self.cli.ISCSIConnection.discoverSendTargets(
+                host=address,
+                port=port,
+                user=user,
+                password=password,
+            )
+            self.logger.debug(targets)
+        except ServerError as e:
+            if e.params['user']:
+                e.params['password'] = '******'
+            raise RuntimeError(str(e))
+
         full_target_template = (
             '^(?P<portal_hostname>[\w\d\-\.]+):(?P<portal_port>\d+),'
-            '(?P<tgpt>-?\d+) (?P<iqn>[\w\d\-\.:]+)$'
+            '(?P<tpgt>-?\d+) (?P<iqn>[\w\d\-\.:]+)$'
         )
         full_target_template_re = re.compile(full_target_template)
-        found = []
-        for t in targets['items']:
+        found = {}
+        for t in targets:
             m = full_target_template_re.match(t)
             mg = m.groupdict()
-            if (
-                mg['portal_hostname'] == address and
-                mg['portal_port'] == port
-            ):
-                found.append(mg)
+            if mg['iqn'] not in found:
+                found[mg['iqn']] = {}
+            if mg['tpgt'] not in found[mg['iqn']]:
+                found[mg['iqn']][mg['tpgt']] = []
+            found[mg['iqn']][mg['tpgt']].append(
+                {
+                    'ip': mg['portal_hostname'],
+                    'port': mg['portal_port']
+                }
+            )
         self.logger.debug('found: {f}'.format(f=found))
         return found
 
-    def _iscsi_get_lun_list(self, ip, port, user, password, iqn):
+    def _iscsi_get_lun_list(
+        self,
+        ip_port_list,
+        tpgt,
+        user,
+        password,
+        iqn
+    ):
         retry = self._MAXRETRY
         iscsi_lun_list = []
         for _try in range(0, retry):
-            devices = self.cli.getDeviceList(
-                ohostedcons.VDSMConstants.ISCSI_DOMAIN
-            )
-            self.logger.debug(devices)
-            if devices['status']['code'] != 0:
-                raise RuntimeError(devices['status']['message'])
-            if 'items' in devices:
-                for device in devices['items']:
-                    for path in device['pathlist']:
-                        if path['iqn'] == iqn:
-                            if device not in iscsi_lun_list:
-                                iscsi_lun_list.append(device)
+            self.logger.info('Connecting to the storage server')
+            connectionParams = []
+            for entry in ip_port_list:
+                connectionParams.append(
+                    {
+                        'connection': entry['ip'],
+                        'iqn': iqn,
+                        'user': user,
+                        'password': password,
+                        'port': entry['port'],
+                        'id': ohostedcons.Const.BLANK_UUID,
+                        'tpgt': tpgt,
+                    }
+                )
+            try:
+                self.cli.StoragePool.connectStorageServer(
+                    storagepoolID=ohostedcons.Const.BLANK_UUID,
+                    domainType=ohostedcons.VDSMConstants.ISCSI_DOMAIN,
+                    connectionParams=connectionParams,
+                )
+            except ServerError as e:
+                raise RuntimeError(str(e))
+
+            try:
+                devices = self.cli.Host.getDeviceList(
+                    storageType=ohostedcons.VDSMConstants.ISCSI_DOMAIN
+                )
+                self.logger.debug(devices)
+            except ServerError as e:
+                self.logger.debug(devices)
+                raise RuntimeError(str(e))
+            for device in devices:
+                for path in device['pathlist']:
+                    if path['iqn'] == iqn:
+                        if device not in iscsi_lun_list:
+                            iscsi_lun_list.append(device)
+
             if iscsi_lun_list:
                 break
 
-            self.logger.info('Discovering iSCSI node')
-            self._iscsi_discovery(
-                ip,
-                port,
-                user,
-                password,
-            )
-            self.logger.info('Connecting to the storage server')
-            connectionParams = [
-                {
-                    'connection': ip,
-                    'iqn': iqn,
-                    # FIXME!
-                    'portal': '0',
-                    'user': user,
-                    'password': password,
-                    'port': port,
-                    'id': ohostedcons.Const.BLANK_UUID,
-                }
-            ]
-            res = self.cli.connectStorageServer(
-                storagepoolID=ohostedcons.Const.BLANK_UUID,
-                domainType=ohostedcons.VDSMConstants.ISCSI_DOMAIN,
-                connectionParams=connectionParams,
-            )
-            if res['status']['code'] != 0:
-                raise RuntimeError(res['status']['message'])
             retry -= 1
             time.sleep(self._RETRY_DELAY)
         else:
@@ -393,19 +449,28 @@ class Plugin(plugin.PluginBase):
 
     def _fc_get_lun_list(self):
         fc_lun_list = []
-        devices = self.cli.getDeviceList(
-            ohostedcons.VDSMConstants.FC_DOMAIN
-        )
-        self.logger.debug(devices)
-        if devices['status']['code'] != 0:
-            raise RuntimeError(devices['status']['message'])
-        for device in devices['items']:
+        try:
+            devices = self.cli.Host.getDeviceList(
+                storageType=ohostedcons.VDSMConstants.FC_DOMAIN
+            )
+            self.logger.debug(devices)
+        except ServerError as e:
+            raise RuntimeError(str(e))
+
+        for device in devices:
             fc_lun_list.append(device)
         return fc_lun_list
 
-    def _iscsi_get_device(self, ip, port, user, password, iqn, lunGUID):
+    def _iscsi_get_device(
+            self,
+            ip_port_list,
+            tpgt,
+            user, password,
+            iqn,
+            lunGUID
+    ):
         available_luns = self._iscsi_get_lun_list(
-            ip, port, user, password, iqn
+            ip_port_list, tpgt, user, password, iqn
         )
         for iscsi_device in available_luns:
             if iscsi_device['GUID'] == lunGUID:
@@ -419,11 +484,21 @@ class Plugin(plugin.PluginBase):
                     return fc_device
         return None
 
-    def _validate_domain(self, domainType, target, lunGUID):
+    def _validate_domain(self, domainType, target, tpgt, lunGUID):
         if domainType == ohostedcons.DomainTypes.ISCSI:
+            ip_port_list = [
+                {'ip': x[0], 'port': x[1]} for x in zip(
+                    self.environment[
+                        ohostedcons.StorageEnv.ISCSI_IP_ADDR
+                    ].split(','),
+                    self.environment[
+                        ohostedcons.StorageEnv.ISCSI_PORT
+                    ].split(',')
+                )
+            ]
             device = self._iscsi_get_device(
-                ip=self.environment[ohostedcons.StorageEnv.ISCSI_IP_ADDR],
-                port=self.environment[ohostedcons.StorageEnv.ISCSI_PORT],
+                ip_port_list=ip_port_list,
+                tpgt=tpgt,
                 user=self.environment[ohostedcons.StorageEnv.ISCSI_USER],
                 password=self.environment[
                     ohostedcons.StorageEnv.ISCSI_PASSWORD
@@ -563,6 +638,7 @@ class Plugin(plugin.PluginBase):
         lunGUID = None
         valid_lun = False
         target = None
+        valid_targets_dict = {}
         if self.environment[ohostedcons.StorageEnv.MNT_OPTIONS]:
             msg = _(
                 'Custom mount options are not supported on {type} devices.'
@@ -577,7 +653,6 @@ class Plugin(plugin.PluginBase):
             port = None
             user = None
             password = None
-            valid_targets = []
             while not valid_access:
                 address = self._customize_ip_address()
                 port = self._customize_port()
@@ -592,8 +667,7 @@ class Plugin(plugin.PluginBase):
                         user,
                         password,
                     )
-                    valid_targets = [x['iqn'] for x in valid_targets_dict]
-                    if valid_targets:
+                    if valid_targets_dict:
                         valid_access = True
                     else:
                         self.logger.error(_('No valid target'))
@@ -609,16 +683,34 @@ class Plugin(plugin.PluginBase):
 
         while not valid_lun:
             if self.domainType == ohostedcons.DomainTypes.ISCSI:
-                target = self._customize_target(
-                    values=valid_targets,
-                    default=valid_targets[0]
+                target, tpgt = self._customize_target(
+                    values=valid_targets_dict,
                 )
+                self.logger.debug(
+                    "target: {target}, tpgt: {tpgt}".format(
+                        target=target,
+                        tpgt=tpgt,
+                    )
+                )
+                ip_port_list = valid_targets_dict[target][tpgt]
             else:
                 target = None
-            lunGUID = self._customize_lun(self.domainType, target)
+                ip_port_list = None
+                tpgt = None
+            lunGUID = self._customize_lun(
+                self.domainType,
+                target,
+                ip_port_list,
+                tpgt
+            )
             if lunGUID is not None:
                 try:
-                    self._validate_domain(self.domainType, target, lunGUID)
+                    self._validate_domain(
+                        self.domainType,
+                        target,
+                        tpgt,
+                        lunGUID
+                    )
                     valid_lun = True
                 except Exception as e:
                     self.logger.debug('exception', exc_info=True)
@@ -631,6 +723,13 @@ class Plugin(plugin.PluginBase):
                 )
         if self.domainType == ohostedcons.DomainTypes.ISCSI:
             self.environment[ohostedcons.StorageEnv.ISCSI_TARGET] = target
+            self.environment[
+                ohostedcons.StorageEnv.ISCSI_IP_ADDR
+            ] = ','.join([x['ip'] for x in ip_port_list])
+            self.environment[
+                ohostedcons.StorageEnv.ISCSI_PORT
+            ] = ','.join([x['port'] for x in ip_port_list])
+            self.environment[ohostedcons.StorageEnv.ISCSI_PORTAL] = tpgt
         self.environment[ohostedcons.StorageEnv.LUN_ID] = lunGUID
 
     @plugin.event(
@@ -660,66 +759,53 @@ class Plugin(plugin.PluginBase):
             ]:
                 forceVG = True
             self.logger.info(_('Creating Volume Group'))
-            dom = self._create_vg(forceVG)
-            self.logger.debug(dom)
-            if dom['status']['code'] != 0:
-                self.logger.error(
-                    _(
-                        'Error creating Volume Group: {message}'
-                    ).format(
-                        message=dom['status']['message']
+
+            vg_uuid = None
+            while vg_uuid is None:
+                try:
+                    vg_uuid = self.cli.LVMVolumeGroup.create(
+                        name=self.environment[ohostedcons.StorageEnv.SD_UUID],
+                        devlist=[
+                            self.environment[
+                                ohostedcons.StorageEnv.LUN_ID
+                            ],
+                        ],
+                        force=forceVG,
                     )
-                )
-                if not forceVG:
-                    # eventually retry forcing VG creation on dirty storage
-                    self._customize_forcecreatevg()
-                    if not self.environment[
-                        ohostedcons.StorageEnv.FORCE_CREATEVG
-                    ]:
-                        raise RuntimeError(dom['status']['message'])
-                    else:
-                        forceVG = True
-                        dom = self._create_vg(forceVG)
-                        self.logger.debug(dom)
-                        if dom['status']['code'] != 0:
-                            self.logger.error(
-                                _(
-                                    'Error creating Volume Group: {message}'
-                                ).format(
-                                    message=dom['status']['message']
-                                )
-                            )
-                            raise RuntimeError(dom['status']['message'])
-                else:
-                    raise RuntimeError(dom['status']['message'])
+                    self.logger.debug(vg_uuid)
+
+                except ServerError as e:
+                    self.logger.error(
+                        _(
+                            'Error creating Volume Group: {message}'
+                        ).format(message=str(e))
+                    )
+
+                    if not forceVG:
+                        # eventually retry forcing VG creation on dirty storage
+                        self._customize_forcecreatevg()
+                        if self.environment[
+                            ohostedcons.StorageEnv.FORCE_CREATEVG
+                        ]:
+                            forceVG = True
+                            # Retry VG creation
+                            continue
+
+                    raise RuntimeError(str(e))
+
             self.environment[
                 ohostedcons.StorageEnv.VG_UUID
-            ] = dom['status']['message']
+            ] = vg_uuid
 
-        vginfo = self.cli.getVGInfo(
-            self.environment[ohostedcons.StorageEnv.VG_UUID]
-        )
-        self.logger.debug(vginfo)
-        if vginfo['status']['code'] != 0:
-            raise RuntimeError(vginfo['status']['message'])
-        if (
-            self.domainType == ohostedcons.DomainTypes.ISCSI and
-            self.environment[ohostedcons.StorageEnv.ISCSI_PORTAL] is None
-        ):
-            try:
-                for pv in vginfo['pvlist']:
-                    for path in pv['pathlist']:
-                        if path['iqn'] == self.environment[
-                            ohostedcons.StorageEnv.ISCSI_TARGET
-                        ]:
-                            self.environment[
-                                ohostedcons.StorageEnv.ISCSI_PORTAL
-                            ] = path['portal']
-                            break
-            except (ValueError, KeyError) as e:
-                self.logger.debug('exception', exc_info=True)
-                self.logger.error(_('Cannot detect iSCSI portal'))
-                raise e
+        try:
+            vginfo = self.cli.LVMVolumeGroup.getInfo(
+                lvmvolumegroupID=self.environment[
+                    ohostedcons.StorageEnv.VG_UUID
+                ]
+            )
+            self.logger.debug(vginfo)
+        except ServerError as e:
+            raise RuntimeError(str(e))
 
 
 # vim: expandtab tabstop=4 shiftwidth=4
